package nl.iobyte.themepark.api.config.objects;

import nl.iobyte.themepark.utils.VersionComparator;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;

public class ConfigurationUpdater {

    private final Configuration config;

    public ConfigurationUpdater(Configuration config) {
        this.config = config;
    }

    public String getCurrentVersion() {
        if(!config.contains("version"))
            return "0.0";

        return config.getString("version");
    }

    public void update() {
        if(config.getFileName().endsWith(".json")) {
            return;
        }

        InputStream resource = config.getPlugin().getResource(config.getFileName());
        if(resource == null)
            return;

        FileConfiguration data;
        try(InputStreamReader reader = new InputStreamReader(resource, StandardCharsets.UTF_8)) {
            data = YamlConfiguration.loadConfiguration(reader);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        String saved_version = "1.0";
        if(data.contains("version"))
            saved_version = data.getString("version");

        VersionComparator comparator = new VersionComparator();
        int i = comparator.compare(saved_version, getCurrentVersion());
        if(i <= 0)
            return;

        insertChanges(data.getConfigurationSection(""), "");
        config.set("version", saved_version);
        config.save();
    }

    private void insertChanges(ConfigurationSection section, String root) {
        if(section == null || section.getKeys(false).isEmpty())
            return;

        String path;
        for(String str : section.getKeys(false)) {
            path = root.equals("") ? str : root + "." + str;
            ConfigurationSection s = section.getConfigurationSection(str);
            if(s != null) {
                insertChanges(s, path);
                continue;
            }

            if(config.contains(path))
                continue;

            config.set(path, section.get(str));
        }
    }

}
