package nl.iobyte.themepark.api.database.objects.types;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.database.objects.Database;
import nl.iobyte.themepark.logger.ThemeParkLogger;
import java.sql.Connection;
import java.sql.SQLException;

public class MySQLDatabase extends Database {

    private final String dbName;
    private final HikariDataSource source;

    public MySQLDatabase(String url, String host, int port, String dbName, String username, String password) {
        if(url == null || url.isEmpty())
            url = "jdbc:mysql://%host%:%port%/%database%?useSSL=false";

        this.dbName = dbName;
        HikariConfig config = new HikariConfig();
        config.setPoolName(ThemePark.getInstance().getName());
        config.setUsername(username);
        config.setPassword(password);
        config.setDriverClassName("com.mysql.jdbc.Driver");
        config.setMaximumPoolSize(5);
        config.setJdbcUrl(url.replace("%host%", host).replace("%port%", String.valueOf(port)).replace("%database%", dbName));
        source = new HikariDataSource(config);
    }

    /**
     * Get Connection
     * @return Connection
     */
    public Connection getConnection() throws SQLException {
        return source.getConnection();
    }

    /**
     * Close connection
     */
    public void closeConnection() {
        ThemeParkLogger.toConsole("Closing the database connection for " + dbName);
        source.close();
    }

}