package nl.iobyte.themepark.api.event;

import org.bukkit.Bukkit;
import org.bukkit.event.Event;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginManager;

public class EventDispatcher {

    private final Plugin plugin;

    public EventDispatcher(Plugin plugin) {
        this.plugin = plugin;
    }

    /**
     * Dispatch Event without Callback
     * @param event Event
     */
    public void call(Event event) {
        call(event, null);
    }

    /**
     * Dispatch Event with Callback
     * @param event Event
     * @param callback Runnable
     */
    public void call(Event event, Runnable callback) {
        PluginManager pm = Bukkit.getPluginManager();
        if(Bukkit.isPrimaryThread()) {
            pm.callEvent(event);
            if(callback != null)
                callback.run();
        } else {
            Bukkit.getScheduler().runTask(plugin, () -> {
                pm.callEvent(event);
                if(callback != null)
                    Bukkit.getScheduler().runTaskAsynchronously(plugin, callback);
            });
        }
    }

}
