package nl.iobyte.themepark.api.menu.objects;

import com.cryptomorin.xseries.XMaterial;
import nl.iobyte.menuapi.MenuAPI;
import nl.iobyte.menuapi.action.HandlerAction;
import nl.iobyte.menuapi.basic.Menu;
import nl.iobyte.menuapi.interfaces.IActionHandler;
import nl.iobyte.menuapi.item.ItemBuilder;
import nl.iobyte.menuapi.item.MenuItem;
import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.config.ConfigurationManager;
import nl.iobyte.themepark.api.config.enums.StorageKey;
import nl.iobyte.themepark.api.config.enums.StorageLocation;
import nl.iobyte.themepark.api.menu.objects.handlers.CommandMenuHandler;
import nl.iobyte.themepark.api.menu.objects.handlers.JsonMessageMenuHandler;
import nl.iobyte.themepark.api.menu.objects.handlers.PlainMessageMenuHandler;
import org.bukkit.Material;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.Player;

public class MainMenu {

    private Menu menu;

    public void load() {
        ConfigurationManager manager = ThemePark.getInstance().getAPI().getConfigurationManager();

        //Prepare Menu
        menu = new Menu(manager.getString(StorageKey.MENU_TITLE_MAIN), manager.getInt(StorageKey.MENU_SIZE_MAIN));

        //Load Items
        loadItems(manager);

        //Build Menu
        menu.build();
    }

    private void loadItems(ConfigurationManager manager) {
        //See if any items exist
        if(!manager.contains(StorageLocation.MENU, "items"))
            return;

        ConfigurationSection section = manager.getSection(StorageLocation.MENU, "items");
        if(section.getKeys(false).isEmpty())
            return;

        //Load items
        String path;
        ItemBuilder builder;
        MenuItem item;
        IActionHandler handler;
        for(String id : section.getKeys(false)) {
            path = "items."+id;

            int slot = manager.getInt(StorageLocation.MENU, path+".slot");
            if(slot < 0 || slot >= menu.getSize())
                continue;

            String name = manager.getString(StorageLocation.MENU, path+".name");
            if(name == null || name.equals(""))
                name = " ";

            XMaterial material = XMaterial.matchXMaterial(manager.getString(StorageLocation.MENU, path+".material")).orElse(null);
            if(material == null)
                continue;

            Material m = material.parseMaterial();
            if(m == null)
                continue;

           int amount = manager.getInt(StorageLocation.MENU, path+".amount");
            if(amount < 1)
                amount = 1;

            short data = Short.parseShort(manager.getString(StorageLocation.MENU, path+".data"));
            if(data < 0)
                continue;

            builder = new ItemBuilder(m, amount, data);
            builder.setName(name);

            String lore = manager.getString(StorageLocation.MENU, path+".lore");
            if(lore == null || lore.equals("")) {
                builder.setLore(manager.getStringList(StorageLocation.MENU, path+".lore"));
            } else {
                builder.setLore(lore);
            }

            item = new MenuItem(
                    builder.getItem(),
                    true
            );

            handler = getHandler(manager, path);
            if(handler != null) {
                item.addActions(handler);
            } else {
                item.addActions(new HandlerAction(
                        manager.getString(StorageLocation.MENU, path+".action")
                ));
            }

            menu.setItem(
                    slot,
                    item
            );
        }
    }

    private IActionHandler getHandler(ConfigurationManager manager, String path) {
        String action = manager.getString(StorageLocation.MENU, path+".action");
        if(action == null || action.isEmpty())
            return null;

        if(MenuAPI.hasActionHandler(action))
            return MenuAPI.getActionHandler(action);

        switch (action.toUpperCase()) {
            case "COMMAND":
            case "BUNGEE_COMMAND":
                return new CommandMenuHandler(manager.getString(StorageLocation.MENU, path+".command"));
            case "JSON_MESSAGE":
                return new JsonMessageMenuHandler(manager.getString(StorageLocation.MENU, path+".message"));
            case "PLAIN_MESSAGE":
                return new PlainMessageMenuHandler(manager.getString(StorageLocation.MENU, path+".message"));
            default:
                return null;
        }
    }

    public void open(Player player) {
        menu.open(player);
    }

}
