package nl.iobyte.themepark.api.ridecount.objects;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.ThemeParkAPI;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themepark.api.database.objects.Database;
import nl.iobyte.themepark.scheduler.ThemeParkScheduler;
import java.util.*;
import java.util.concurrent.CompletableFuture;

public class TopData {

    public CompletableFuture<HashMap<UUID, Integer>> getDay(Attraction attraction) {
        return get(attraction, 0);
    }

    public CompletableFuture<HashMap<UUID, Integer>> getWeek(Attraction attraction) {
        return get(attraction, 1);
    }

    public CompletableFuture<HashMap<UUID, Integer>> getMonth(Attraction attraction) {
        return get(attraction, 2);
    }

    public CompletableFuture<HashMap<UUID, Integer>> getYear(Attraction attraction) {
        return get(attraction, 3);
    }

    private CompletableFuture<HashMap<UUID, Integer>> get(Attraction attraction, int i) {
        CompletableFuture<HashMap<UUID, Integer>> fc = new CompletableFuture<>();
        if(attraction == null) {
            fc.complete(null);
            return fc;
        }

        ThemeParkAPI api = ThemePark.getInstance().getAPI();
        if(!api.getAttractionService().hasAttraction(attraction.getID())) {
            fc.complete(null);
            return fc;
        }

        Database db = api.getDatabaseService().getDatabase("local");
        if(db == null) {
            fc.complete(null);
            return fc;
        }

        ThemeParkScheduler.runAsync(() -> {
            Calendar calendar = new GregorianCalendar();
            calendar.setTime(new Date());

            HashMap<Integer, Object> map = new HashMap<>();
            map.put(1, attraction.getID());

            String timeSort;
            switch (i) {
                case 0:
                    timeSort = "day=? AND year=?";
                    map.put(2, calendar.get(Calendar.DAY_OF_YEAR));
                    map.put(3, calendar.get(Calendar.YEAR));
                    break;
                case 1:
                    timeSort = "week=? AND year=?";
                    map.put(2, calendar.get(Calendar.WEEK_OF_YEAR));
                    map.put(3, calendar.get(Calendar.YEAR));
                    break;
                case 2:
                    timeSort = "month=? AND year=?";
                    map.put(2, calendar.get(Calendar.MONTH) + 1);
                    map.put(3, calendar.get(Calendar.YEAR));
                    break;
                default:
                    timeSort = "year=?";
                    map.put(2, calendar.get(Calendar.YEAR));
                    break;
            }

            ArrayList<Map<String, Object>> results = db.executeQuery("SELECT uuid, SUM(count) AS count FROM counts WHERE attraction_id=? AND "+timeSort+" GROUP BY uuid ORDER BY count DESC LIMIT 3", map);
            if(results == null || results.isEmpty()) {
                fc.complete(null);
                return;
            }

            HashMap<UUID, Integer> top = new HashMap<>();
            for(Map<String, Object> m : results) {
                UUID uuid = UUID.fromString((String) m.get("uuid"));
                int points = (int) m.get("count");
                if(points < 0)
                    continue;

                top.put(uuid, points);
            }

            fc.complete(top);
        });
        return fc;
    }

    public CompletableFuture<HashMap<UUID, Integer>> getTotal(Attraction attraction) {
        CompletableFuture<HashMap<UUID, Integer>> fc = new CompletableFuture<>();
        if(attraction == null) {
            fc.complete(null);
            return fc;
        }

        ThemeParkAPI api = ThemePark.getInstance().getAPI();
        if(!api.getAttractionService().hasAttraction(attraction.getID())) {
            fc.complete(null);
            return fc;
        }

        Database db = api.getDatabaseService().getDatabase("local");
        if(db == null) {
            fc.complete(null);
            return fc;
        }

        ThemeParkScheduler.runAsync(() -> {
            HashMap<Integer, Object> map = new HashMap<>();
            map.put(1, attraction.getID());

            ArrayList<Map<String, Object>> results = db.executeQuery("SELECT uuid, SUM(count) AS count FROM counts WHERE attraction_id=? GROUP BY uuid ORDER BY count DESC LIMIT 3", map);
            if(results == null || results.isEmpty()) {
                fc.complete(null);
                return;
            }

            HashMap<UUID, Integer> top = new HashMap<>();
            for(Map<String, Object> m : results) {
                UUID uuid = UUID.fromString((String) m.get("uuid"));
                int points = (int) m.get("count");
                if(points < 0)
                    continue;

                top.put(uuid, points);
            }

            fc.complete(top);
        });
        return fc;
    }

}
