package nl.iobyte.themepark.commands.subcommands;

import nl.iobyte.commandapi.CommandFactory;
import nl.iobyte.commandapi.arguments.number.IntegerArgument;
import nl.iobyte.commandapi.objects.SubCommand;
import nl.iobyte.themepark.api.message.Text;
import nl.iobyte.commandapi.interfaces.ICommandExecutor;
import java.util.List;

public class HelpCommand extends SubCommand {

    private final CommandFactory factory;

    public HelpCommand(CommandFactory factory) {
        super(new String[]{"help"});

        this.factory = factory;
        addSyntax("/"+factory.getName()+" help");
        addSyntax("/"+factory.getName()+" help <page>")
                .addArgument(new IntegerArgument());
    }

    //Send ICommandExecutor list of commands it has access to
    public void onCommand(ICommandExecutor sender, List<Object> list, int i) {
        List<SubCommand> commands = factory.getApplicableSubCommands(sender);
        if(commands.size() <= 5) {
            sendSinglePage(sender, commands);
            return;
        }

        sendMultiPage(sender, commands, list, i);
    }

    private void sendSinglePage(ICommandExecutor sender, List<SubCommand> commands) {
        sender.sendMessage(Text.color("&f&l>==== &6&lThemePark &l&f====<"));
        for (SubCommand command : commands)
            sender.sendMessage(Text.color("&f" + command.getApplicableSyntaxList(sender).get(0).getUsage()));

        sender.sendMessage(Text.color("&f&l>==== &6&lThemePark &l&f====<"));
    }

    private void sendMultiPage(ICommandExecutor sender, List<SubCommand> commands, List<Object> list, int i) {
        int page = 1;
        int pages = (int) Math.ceil(((double) commands.size()) / 5);
        if(i == 1)
            page = (Integer) list.get(0);

        if(page < 1 || page > pages) {
            sender.sendMessage(Text.color("&6&lThemePark &f➢ Page &6"+page+" &fdoesn't exist"));
            return;
        }

        page--;
        int start = page * 5;
        int end = page * 5 + 5;
        if(end > commands.size())
            end = commands.size();

        sender.sendMessage(Text.color("&f&l>==== &6&lThemePark &f(&6"+(page + 1)+"&f/&6"+pages+"&f) &l&f====<"));
        for(i = start; i < end; i++)
            sender.sendMessage(Text.color("&f" + commands.get(i).getApplicableSyntaxList(sender).get(0).getUsage()));

        sender.sendMessage(Text.color("&f&l>==== &6&lThemePark &l&f====<"));
    }

}
