package nl.iobyte.themepark.commands.subcommands.attraction;

import com.google.common.collect.Iterables;
import nl.iobyte.commandapi.arguments.number.IntegerArgument;
import nl.iobyte.commandapi.interfaces.ICommandExecutor;
import nl.iobyte.commandapi.objects.SubCommand;
import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themepark.api.attraction.objects.Region;
import nl.iobyte.themepark.api.message.Text;
import nl.iobyte.themepark.commands.arguments.RegionArgument;
import java.util.Collection;
import java.util.List;

public class AttractionListCommand extends SubCommand {

    public AttractionListCommand(String cmd)  {
        super("themepark.admin", "attraction", "list");

        addSyntax("/"+cmd+" attraction list");

        addSyntax("/"+cmd+" attraction list <page>")
                .addArgument(new IntegerArgument());

        addSyntax("/"+cmd+" attraction list <region>")
                .addArgument(new RegionArgument());

        addSyntax("/"+cmd+" attraction list <region> <page>")
                .addArgument(new RegionArgument())
                .addArgument(new IntegerArgument());
    }

    
    public void onCommand(ICommandExecutor  sender, List<Object> list, int i) {
        int page = 1;
        String title = "&6&lThemePark";
        Collection<Attraction> attractions;
        if(i < 2) {
            if(i == 1)
                page = (Integer) list.get(0);

            attractions = ThemePark.getInstance().getAPI().getAttractionService().getAttractions().values();
        } else {
            if(i == 3)
                page = (Integer) list.get(1);

            Region region = (Region) list.get(0);
            title = region.getName();
            attractions = region.getAttractions().values();
        }

        if(attractions.size() <= 5) {
            sendSinglePage(sender, title, attractions);
        } else {
            sendMultiPage(sender, page, title, attractions);
        }
    }

    private void sendSinglePage(ICommandExecutor sender, String title, Collection<Attraction> attractions) {
        sender.sendMessage(Text.color("&f&l>==== &r"+title+" &l&f====<"));
        if(attractions.isEmpty()) {
            sender.sendMessage(Text.color("&6No attractions found"));
        } else {
            for (Attraction attraction : attractions)
                sender.sendMessage(Text.color(
                        "&6ID: &f" + attraction.getID()
                                + " &6Region: &f" + attraction.getRegionID()
                                + " &6Name: &r" + attraction.getName()
                                + " &6Status: &r" + attraction.getStatus().getColor() + attraction.getStatus().getName()));
        }

        sender.sendMessage(Text.color("&f&l>==== &r"+title+" &l&f====<"));
    }

    private void sendMultiPage(ICommandExecutor sender, int page, String title, Collection<Attraction> attractions) {
        int pages = (int) Math.ceil(((double) attractions.size()) / 5);
        if(page < 1 || page > pages) {
            sender.sendMessage(Text.color("&6&lThemePark &f➢ Page &6"+page+" &fdoesn't exist"));
            return;
        }

        int i = 0;
        for(List<Attraction> lr : Iterables.partition(attractions, 5)) {
            i++;
            if(i != page)
                continue;

            sender.sendMessage(Text.color("&f&l>==== &r"+title+" &f(&6"+page+"&f/&6"+pages+"&f) &l&f====<"));
            for(Attraction attraction : lr)
                sender.sendMessage(Text.color(
                        "&6ID: &f"+attraction.getID()
                                +" &6Region: &f"+attraction.getRegionID()
                                +" &6Name: &r"+attraction.getName()
                                +" &6Status: &r"+attraction.getStatus().getColor()+attraction.getStatus().getName()));

            sender.sendMessage(Text.color("&f&l>==== &r"+title+" &l&f====<"));
            break;
        }
    }

}
