package nl.iobyte.themepark.listeners;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.ThemeParkAPI;
import nl.iobyte.themepark.api.attraction.AttractionService;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themepark.api.attraction.objects.Region;
import nl.iobyte.themepark.api.config.objects.Configuration;
import nl.iobyte.themepark.api.event.attraction.*;
import nl.iobyte.themepark.utils.LocationUtil;
import nl.iobyte.themepark.utils.StatusMessage;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import java.util.HashMap;

public class AttractionListener implements Listener {

    @EventHandler
    public void onCreate(AttractionCreateEvent e) {
        ThemeParkAPI api = ThemePark.getInstance().getAPI();
        Attraction attraction = e.getAttraction();
        Region region = api.getAttractionService().getRegion(
                attraction.getRegionID()
        );

        //Add to Region's config
        String path = "attractions."+attraction.getID();
        Configuration config = region.getConfiguration();
        config.set(path+".name", attraction.getName());
        config.set(path+".cover", attraction.getCover());
        config.set(path+".status", attraction.getStatus().toString());
        config.set(path+".location", LocationUtil.toString(attraction.getLocation()));

        //Update Menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().load();

        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "INSERT IGNORE INTO attractions(id, region_id, name, cover, status_id) VALUES (?,?,?,?,?)",
                new HashMap<>() {{
                    put(1, attraction.getID());
                    put(2, attraction.getRegionID());
                    put(3, attraction.getName());
                    put(4, attraction.getCover());
                    put(5, attraction.getStatus().getID());
                }}
        );
    }

    @EventHandler
    public void onRegionChange(AttractionRegionChangeEvent e) {
        Attraction attraction = e.getAttraction();
        AttractionService service = ThemePark.getInstance().getAPI().getAttractionService();

        //Remove from old Region
        Region region = service.getRegion(e.getOld());
        region.removeAttraction(attraction.getID());

        //Remove from old Region's config
        Configuration config = region.getConfiguration();
        config.set("attractions."+attraction.getID(), null);

        //Add to new Region
        region = service.getRegion(e.getCurrent());
        region.addAttraction(attraction);

        //Add to new Region's config
        String path = "attractions."+attraction.getID();
        config = region.getConfiguration();
        config.set(path+".name", attraction.getName());
        config.set(path+".cover", attraction.getCover());
        config.set(path+".status", attraction.getStatus().toString());
        config.set(path+".location", LocationUtil.toString(attraction.getLocation()));

        //Update Menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().load();

        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "UPDATE attractions SET region_id=? WHERE id=?",
                new HashMap<Integer, Object>() {{
                    put(1, e.getCurrent());
                    put(2, attraction.getID());
                }}
        );
    }

    @EventHandler
    public void onNameChange(AttractionNameChangeEvent e) {
        Attraction attraction = e.getAttraction();
        Region region = ThemePark.getInstance().getAPI().getAttractionService().getRegion(
                attraction.getRegionID()
        );

        //Change name in Region's config
        Configuration config = region.getConfiguration();
        config.set("attractions."+attraction.getID()+".name", e.getCurrent());

        //Update Menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().updateAttraction(attraction);

        //Update Sign
        ThemePark.getInstance().getAPI().getSignManager().update(attraction);

        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "UPDATE attractions SET name=? WHERE id=?",
                new HashMap<>() {{
                    put(1, e.getCurrent());
                    put(2, attraction.getID());
                }}
        );
    }

    @EventHandler
    public void onCoverChange(AttractionCoverChangeEvent e) {
        Attraction attraction = e.getAttraction();
        Region region = ThemePark.getInstance().getAPI().getAttractionService().getRegion(
                attraction.getRegionID()
        );

        //Change cover in Region's config
        Configuration config = region.getConfiguration();
        config.set("attractions."+attraction.getID()+".cover", e.getCurrent());

        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "UPDATE attractions SET cover=? WHERE id=?",
                new HashMap<Integer, Object>() {{
                    put(1, e.getCurrent());
                    put(2, attraction.getID());
                }}
        );
    }

    @EventHandler
    public void onStatusChange(AttractionStatusChangeEvent e) {
        Attraction attraction = e.getAttraction();
        Region region = ThemePark.getInstance().getAPI().getAttractionService().getRegion(
                attraction.getRegionID()
        );

        //Change status in Region's config
        Configuration config = region.getConfiguration();
        config.set("attractions."+attraction.getID()+".status", e.getCurrent().toString());

        //Update Menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().updateAttraction(attraction);

        //Update Sign
        ThemePark.getInstance().getAPI().getSignManager().update(attraction);

        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "UPDATE attractions SET status_id=? WHERE id=?",
                new HashMap<>() {{
                    put(1, e.getCurrent().getID());
                    put(2, attraction.getID());
                }}
        );

        //Send Message
        StatusMessage.broadcast(attraction);
    }

    @EventHandler
    public void onLocationChange(AttractionLocationChangeEvent e) {
        ThemeParkAPI api = ThemePark.getInstance().getAPI();
        Attraction attraction = e.getAttraction();
        Region region = api.getAttractionService().getRegion(
                attraction.getRegionID()
        );

        //Change location in Region's config
        Configuration config = region.getConfiguration();
        config.set("attractions."+attraction.getID()+".location", LocationUtil.toString(e.getCurrent()));
    }

    @EventHandler
    public void onRemove(AttractionRemoveEvent e) {
        ThemeParkAPI api = ThemePark.getInstance().getAPI();
        Attraction attraction = e.getAttraction();
        Region region = api.getAttractionService().getRegion(
                attraction.getRegionID()
        );

        //Remove from Region's config
        Configuration config = region.getConfiguration();
        config.set("attractions."+attraction.getID(), null);

        //Update Menu
        api.getMenuService().getStatusMenu().load();

        //Update Sign
        ThemePark.getInstance().getAPI().getSignManager().remove(attraction);

        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "DELETE attractions, ridecounts FROM attractions LEFT JOIN attractions.id=ridecounts.attraction_id WHERE attractions.id=?",
                new HashMap<>() {{
                    put(1, attraction.getID());
                }}
        );
    }

}
