package nl.iobyte.themepark.listeners;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themepark.api.config.enums.StorageKey;
import nl.iobyte.themepark.api.database.objects.types.SQLiteDatabase;
import nl.iobyte.themepark.api.event.ridecount.RideCountAddEvent;
import nl.iobyte.themepark.api.event.ridecount.RideCountTotalTypeChangeEvent;
import nl.iobyte.themepark.api.message.MessageKey;
import nl.iobyte.themepark.api.message.Text;
import nl.iobyte.themepark.api.ridecount.objects.RideCount;
import nl.iobyte.themepark.scheduler.ThemeParkScheduler;
import nl.iobyte.themepark.utils.VersionComparator;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import java.util.HashMap;

public class RideCountListener implements Listener {

    @EventHandler
    public void onAdd(RideCountAddEvent e) {
        RideCount rc = e.getRideCount();
        ThemeParkScheduler.runAsync(() -> {
            VersionComparator vc = new VersionComparator();
            int i = vc.compare(
                    ((SQLiteDatabase) ThemePark.getInstance().getAPI().getDatabaseService().getDatabase("local")).getVersion(),
                    "3.24.0"
            );

            if(i >= 0) {
                ThemePark.getInstance().getAPI().getDatabaseService().execute(
                        "local",
                        "INSERT INTO counts(uuid, attraction_id, year, month, week, day, count) VALUES (?,?,?,?,?,?,?) ON CONFLICT(uuid, attraction_id, year, day)  DO UPDATE SET count=?",
                        new HashMap<>() {{
                            put(1, rc.getUUID().toString());
                            put(2, rc.getAttractionID());
                            put(3, rc.getYear());
                            put(4, rc.getMonth());
                            put(5, rc.getWeek());
                            put(6, rc.getDay());
                            put(7, rc.getCount());
                            put(8, rc.getCount());
                        }}
                );
            } else {
                boolean b = ThemePark.getInstance().getAPI().getDatabaseService().execute(
                        "local",
                        "INSERT OR IGNORE INTO counts(uuid, attraction_id, year, month, week, day, count) VALUES (?,?,?,?,?,?,?)",
                        new HashMap<>() {{
                            put(1, rc.getUUID().toString());
                            put(2, rc.getAttractionID());
                            put(3, rc.getYear());
                            put(4, rc.getMonth());
                            put(5, rc.getWeek());
                            put(6, rc.getDay());
                            put(7, rc.getCount());
                        }}
                );

                if(b)
                    return;

                ThemePark.getInstance().getAPI().getDatabaseService().execute(
                        "local",
                        "UPDATE counts SET count=? WHERE uuid=? AND attraction_id=? AND year=? AND day=?",
                        new HashMap<>() {{
                            put(1, rc.getCount());
                            put(2, rc.getUUID().toString());
                            put(3, rc.getAttractionID());
                            put(4, rc.getYear());
                            put(5, rc.getDay());
                        }}
                );
            }
        });

        Player player = Bukkit.getPlayer(rc.getUUID());
        if(player == null)
            return;

        Attraction attraction = ThemePark.getInstance().getAPI().getAttractionService().getAttraction(rc.getAttractionID());
        if(attraction == null)
            return;

        player.sendMessage(
                MessageKey.RIDECOUNT_ADD.getMessage().replace("%NAME%", Text.strip(attraction.getName()))
                        .replace("%AMOUNT%", String.valueOf(e.getCurrent() - e.getOld()))
                        .replace("%TOTAL%", String.valueOf(rc.getTotalCount()))
        );
    }

    @EventHandler
    public void onChance(RideCountTotalTypeChangeEvent e) {
        ThemePark.getInstance().getAPI().getConfigurationManager().set(StorageKey.RIDECOUNT_TOTAL_TYPE, e.getCurrent());
    }

}
