package nl.iobyte.themepark.listeners;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themepark.api.config.enums.StorageLocation;
import nl.iobyte.themepark.api.event.status.*;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import java.util.HashMap;

public class StatusListener implements Listener {

    @EventHandler
    public void onColorChange(StatusColorChangeEvent e) {
        ThemePark.getInstance().getAPI().getConfigurationManager().set(
                StorageLocation.STATUS_SETTINGS,
                "states."+e.getStatus().toString()+".color",
                e.getCurrent()
        );

        //Update menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().updateStatus(e.getStatus());

        //Update Sign
        for(Attraction attraction : ThemePark.getInstance().getAPI().getAttractionService().getAttractionsByStatus(e.getStatus()).values())
            ThemePark.getInstance().getAPI().getSignManager().update(attraction);
    }

    @EventHandler
    public void onNameChange(StatusNameChangeEvent e) {
        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "UPDATE states SET name=? WHERE id=?",
                new HashMap<>() {{
                    put(1, e.getCurrent());
                    put(2, e.getStatus().getID());
                }}
        );

        ThemePark.getInstance().getAPI().getConfigurationManager().set(
                StorageLocation.STATUS_SETTINGS,
                "states."+e.getStatus().toString()+".name",
                e.getCurrent()
        );

        //Update menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().updateStatus(e.getStatus());

        //Update Sign
        for(Attraction attraction : ThemePark.getInstance().getAPI().getAttractionService().getAttractionsByStatus(e.getStatus()).values())
            ThemePark.getInstance().getAPI().getSignManager().update(attraction);
    }

    @EventHandler
    public void onHexColorChange(StatusHexColorChangeEvent e) {
        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "UPDATE states SET color=? WHERE id=?",
                new HashMap<>() {{
                    put(1, e.getCurrent());
                    put(2, e.getStatus().getID());
                }}
        );

        ThemePark.getInstance().getAPI().getConfigurationManager().set(
                StorageLocation.STATUS_SETTINGS,
                "states."+e.getStatus().toString()+".hex_color",
                e.getCurrent()
        );
    }

    @EventHandler
    public void onCanTeleportChange(StatusTeleportChangeEvent e) {
        ThemePark.getInstance().getAPI().getConfigurationManager().set(
                StorageLocation.STATUS_SETTINGS,
                "states."+e.getStatus().toString()+".teleport",
                e.getCurrent()
        );
    }

    @EventHandler
    public void onMaterialChange(StatusMaterialChangeEvent e) {
        ThemePark.getInstance().getAPI().getConfigurationManager().set(
                StorageLocation.STATUS_SETTINGS,
                "states."+e.getStatus().toString()+".material",
                e.getCurrent().toString()
        );

        //Update menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().updateStatus(e.getStatus());
    }

}
