package nl.iobyte.themepark.scheduler;

import nl.iobyte.themepark.ThemePark;
import org.bukkit.Bukkit;

public class ThemeParkScheduler {

    /**
     * Execute runnable in sync
     * @param r Runnable
     */
    public static void runSync(Runnable r) {
        if(ThemePark.getInstance().isDisabling()) {
            r.run();
            return;
        }

        Bukkit.getScheduler().runTask(ThemePark.getInstance(), r);
    }

    /**
     * Execute runnable in sync after delay of X ticks
     * @param r Runnable
     * @param delay Delay
     */
    public static void runSyncLater(Runnable r, long delay) {
        if(ThemePark.getInstance().isDisabling()) {
            r.run();
            return;
        }

        Bukkit.getScheduler().runTaskLater(ThemePark.getInstance(), r, delay);
    }

    /**
     * Execute runnable in async
     * @param r Runnable
     */
    public static void runAsync(Runnable r) {
        if(ThemePark.getInstance().isDisabling()) {
            r.run();
            return;
        }

        Bukkit.getScheduler().runTaskAsynchronously(ThemePark.getInstance(), r);
    }

    /**
     * Execute runnable in async after delay of X ticks
     * @param r Runnable
     * @param delay Delay
     */
    public static void runAsyncLater(Runnable r, long delay) {
        if(ThemePark.getInstance().isDisabling()) {
            r.run();
            return;
        }

        Bukkit.getScheduler().runTaskLaterAsynchronously(ThemePark.getInstance(), r, delay);
    }

}
